/* Faked serial routines */

/*
 * Include the generic headers 
 */
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include <avr/io.h>
#include <avr/interrupt.h>

/*
 * Include the queue functions
 */
#include "queue.h"


/*
 * Standard definitions for code below
 */
#define pdTRUE		( 1 )
#define pdFALSE		( 0 )


/*
 * Global variable definitions
 */
static xQueueHandle xCharsForTx; 
extern uint8_t      serPortsEnabled;


/*---------------------------------------------------------------------------*/


void send( char* pcOutBuf, char cBufLen )
{
	while ( (pcOutBuf != NULL) && (cBufLen > 0) )
	{
		/* Return false if after the block time there is no room on the Tx queue. */
//		if( xQueueSend( xCharsForTx[ (portSHORT) PortNum ], pcOutBuf, xBlockTime ) != pdPASS )
//		{
//			return pdFAIL;
//		}
		
		pcOutBuf++;
		cBufLen--;
	}

	UCSR0B |= _BV( TXEN0  );

	return;
}


/*-- INTERRUPT SERVICE ROUTINES **********************************************/


ISR( SIG_USART0_DATA, ISR_BLOCK )
{
	int8_t cChar = 0;

	/* if this port is not enabled, turn off interrupts */
	if ( !( serPortsEnabled & 1 ) )
	{
		UCSR0B &= ~( _BV( TXEN0  ) | _BV( RXEN0  ) );
		return;
	}

	/* If there is a character in the queue for this port, send it to the port.
	   Otherwise, turn off this interrupt until we have more to send */

	if( xQueueReceiveFromISR( xCharsForTx, &cChar ) == pdTRUE )
	{
		/* Send the next character queued for Tx. */
		UDR0 = cChar;
	}
	else
	{
		/* Queue empty, nothing to send, disable the transmit interrupt. */
		UCSR0B &= ~_BV( TXEN0  );
	}
}

