/* Attempted re-creation of ISR register saving bug - WinAVR 20080430 */

/*
 * Include the generic headers 
 */
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include <avr/io.h>
#include <avr/interrupt.h>

/*
 * Standard definitions for code below
 */
#define pdTRUE		( 1 )
#define pdFALSE		( 0 )

#define pdPASS		( 1 )
#define pdFAIL		( 0 )


/*
 * Definition of the queue used by the scheduler.
 * Items are queued by copy, not reference.
 */
typedef struct QueueDefinition
{
	int8_t *pcHead;			/*< Points to the beginning of the queue storage area. */
	int8_t *pcTail;			/*< Points to the byte at the end of the queue storage area.  Once more byte is allocated than necessary to store the queue items, this is used as a marker. */

	int8_t *pcWriteTo;		/*< Points to the free next place in the storage area. */
	int8_t *pcReadFrom;		/*< Points to the last place that a queued item was read from. */

	volatile uint8_t uxMessagesWaiting;/*< The number of items currently in the queue. */
	uint8_t uxLength;		/*< The length of the queue defined as the number of items it will hold, not the number of bytes. */
	uint8_t uxItemSize;		/*< The size of each items that the queue will hold. */

	int8_t xRxLock;			/*< Stores the number of items received from the queue (removed from the queue) while the queue was locked.  Set to queueUNLOCKED when the queue is not locked. */
	int8_t xTxLock;			/*< Stores the number of items transmitted to the queue (added to the queue) while the queue was locked.  Set to queueUNLOCKED when the queue is not locked. */
} xQUEUE;

typedef xQUEUE * xQueueHandle;



/*
 * Global variable definitions
 */
xQueueHandle xCharsForTx; 
uint8_t serPortsEnabled;


/*---------------------------------------------------------------------------*/

static void prvCopyDataFromQueue( xQUEUE * const pxQueue, const void *pvBuffer )
{
	pxQueue->pcReadFrom += pxQueue->uxItemSize;
	if( pxQueue->pcReadFrom >= pxQueue->pcTail )
	{
		pxQueue->pcReadFrom = pxQueue->pcHead;
	}
//	memcpy( ( void * ) pvBuffer, ( void * ) pxQueue->pcReadFrom, ( unsigned ) pxQueue->uxItemSize );
}

/*---------------------------------------------------------------------------*/

int8_t xQueueReceiveFromISR( xQueueHandle pxQueue, const void * const pvBuffer )
{
	int8_t xReturn;

	/* We cannot block from an ISR, so check there is data available. */
	if( pxQueue->uxMessagesWaiting > ( uint8_t ) 0 )
	{
		prvCopyDataFromQueue( pxQueue, pvBuffer );
		--( pxQueue->uxMessagesWaiting );

		{
			/* Increment the lock count so the task that unlocks the queue
			knows that data was removed while it was locked. */
			++( pxQueue->xRxLock );
		}

		xReturn = pdPASS;
	}
	else
	{
		xReturn = pdFAIL;
	}

	return xReturn;
}

/*---------------------------------------------------------------------------*/


void send( char* pcOutBuf, char cBufLen )
{
	while ( (pcOutBuf != NULL) && (cBufLen > 0) )
	{
		/* Return false if after the block time there is no room on the Tx queue. */
//		if( xQueueSend( xCharsForTx[ (portSHORT) PortNum ], pcOutBuf, xBlockTime ) != pdPASS )
//		{
//			return pdFAIL;
//		}
		
		pcOutBuf++;
		cBufLen--;
	}

	UCSR0B |= _BV( TXEN0  );

	return;
}


/*-- INTERRUPT SERVICE ROUTINES **********************************************/


ISR( SIG_USART0_DATA, ISR_BLOCK )
{
	int8_t cChar = 0;

	/* if this port is not enabled, turn off interrupts */
	if ( !( serPortsEnabled & 1 ) )
	{
		UCSR0B &= ~( _BV( TXEN0  ) | _BV( RXEN0  ) );
		return;
	}

	/* If there is a character in the queue for this port, send it to the port.
	   Otherwise, turn off this interrupt until we have more to send */

	if( xQueueReceiveFromISR( xCharsForTx, &cChar ) == pdTRUE )
	{
		/* Send the next character queued for Tx. */
		UDR0 = cChar;
	}
	else
	{
		/* Queue empty, nothing to send, disable the transmit interrupt. */
		UCSR0B &= ~_BV( TXEN0  );
	}
}


/*---------------------------------------------------------------------------*/

int main( void )
{
	char  buffer[128];
	char* pB;

	pB = buffer;
	serPortsEnabled = 1;

	strcpy(pB, "Hello World!\r\n");

	while (1)
	{
		send(pB, strlen(pB));
	}
}


